# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng.protocol.au680 import Au680Protocol
from driverng.utils import ETB, ETX, ACK, NACK, STX, EOT, byte2log


class DXC700Protocol(Au680Protocol):
    ACCEPTED_END_CHAR = [ETX, ETB]
    count_bad_dialog = False
    count_nack_receive = 0
    count_abort_trame = 0  # On compte le nombre de réponse à abandonner

    def send_trames(self, trame):
        sent = super(Au680Protocol, self).send_trames(trame)

        self.write(EOT)
        return sent

    def recv_trames(self):
        # On lit 1 car
        data = self.read()
        if not data:
            self.logger.warning("<stx> expected, nothing received")
            return False

        # 3 NACK possible avant que le au680 n'accepte plus notre réponse(au prochain STX)
        if data[0] == NACK:
            self.count_bad_dialog = False  # dialog correcte si NACK
            self.count_nack_receive += 1
            self.logger.warning(
                "<stx> expected, (%s) <nack> received", self.count_nack_receive
            )
            if (
                self.count_nack_receive == 3
            ):  # On a recu 3 NACK, le dxc700 n'acceptera plus notre réponse
                self.logger.warning("3 <nack> reçus, envoi annulé")
                self.count_abort_trame += 1
                self.count_nack_receive = 0

            return False

        # On ne se comprend pas, pas la peine d'envoyer une réponse (si on a mal comprit la demande)
        if data[0] != STX:
            self.count_bad_dialog = True
            self.count_nack_receive = 0
            self.logger.warning(
                "<stx> expected but <%s> received", format(byte2log(data))
            )
            return False

        # On travaille maintenant pour la trame suivante
        self.count_nack_receive = 0

        # On lit la suite jusque END_CHAR
        if self.checksum_type == self.CHECKSUM_BCC:
            end_pos = -2
        else:
            end_pos = -1
        trame = data
        trame_ok = False
        while True:
            # il faut faire un read avec un timeout 15
            trame += self.readline(
                break_func=lambda s: s
                and (s[0] == EOT or s[end_pos] in self.ACCEPTED_END_CHAR),
                timeout=30,
            )
            if trame[-1] == EOT:
                break

            # Mauvais checksum, on ne se comprend toujours pas
            if not self.check_trame(trame):
                self.count_bad_dialog = True
                self.write(NACK)
                return False

            self.count_bad_dialog = False

            # on doit envoyer ack
            if self.ack:
                self.write(ACK)

            # on ignore ces trames
            payload = self.get_payload(trame)
            if bytearray(payload[:2]) in (b"DE", b"DB", b"SE", b"RB", b"RE"):
                self.logger.info("data ignored %s", byte2log(payload[:2]))
            else:
                trame_ok = True
                if len(data) > 36 and len(payload) > 36:
                    data += payload[36:]
                else:
                    data = payload
            if trame[-1] == EOT:
                break
            if trame[end_pos] == ETX:
                break
            trame = b""

        return data if trame_ok else b""
