# -*- coding: utf8 -*-
from __future__ import unicode_literals

import copy
import datetime

parser_class = "alinity_mh"


def test_encode_order(self):
    worklist = copy.deepcopy(self.order_worklist)
    message = self.order(
        worklist=worklist,
        context={"control_id": "1521651964637", "placer_order_number": "1234547"},
    )
    expected = """
    MSH|^~\\&|LISApp|MainFacility|AlinityApp|LabFacility|20130715141912||OML^O33^OML_O33|1521651964637|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-28^IHE
    SPM|1|||Serum|||||||P^Patient^HL70369||||||20130513114242
    SAC|||123456||||||||||||||||||||||||||1:10
    ORC|NW||||||||20130603110342
    TQ1|1||||||||S^STAT^HL70485
    OBR||1234547||A^A^99ABT||||||||||||PRESCRIBER
    SPM|1|||Serum|||||||P^Patient^HL70369||||||20130513114242
    SAC|||123456
    ORC|NW||||||||20130603110342
    TQ1|1||||||||S^STAT^HL70485
    OBR||1234547||B^B^99ABT||||||||||||PRESCRIBER
    SPM|1|||Serum|||||||P^Patient^HL70369||||||20130513114242
    SAC|||123456
    ORC|CA||||||||20130603110342
    TQ1|1||||||||S^STAT^HL70485
    OBR||1234547||C^C^99ABT||||||||||||PRESCRIBER
    """
    assert message == expected


def test_decode_ssu(self):
    worklist = self.decode(
        """
    MSH|^~\\&|||||20191128153158+0100||SSU^U03^SSU_U03|813be7d1-1701-414e-8f61-6040ca50d370|P|2.5.1|||NE|AL||UNICODE UTF-8
    EQU|Alinity m^Abbott~m00082^Abbott|20191128153151
    SAC|||614013323105|||||I^Identified^HL70370||A017166|5||||10^Position^99ABT
    SPM|1|||NotParticipating^NotParticipating^HL70487|||||||P^Patient^HL70369
    SAC|||634013323095|||||I^Identified^HL70370||A017166|6||||10^Position^99ABT
    SPM|2|||NotParticipating^NotParticipating^HL70487|||||||P^Patient^HL70369
    SAC|||634013323094|||||I^Identified^HL70370||A017166|7||||10^Position^99ABT
    SPM|3|||NotParticipating^NotParticipating^HL70487|||||||P^Patient^HL70369
    SAC|||634013323084|||||I^Identified^HL70370||A017166|8||||10^Position^99ABT
    SPM|4|||NotParticipating^NotParticipating^HL70487|||||||P^Patient^HL70369
    SAC|||634013323078|||||I^Identified^HL70370||A017166|9||||10^Position^99ABT
    SPM|5|||NotParticipating^NotParticipating^HL70487|||||||P^Patient^HL70369
    SAC|||634013323101|||||I^Identified^HL70370||A017166|10||||10^Position^99ABT
    SPM|6|||NotParticipating^NotParticipating^HL70487|||||||P^Patient^HL70369
    SAC|||634013322029|||||I^Identified^HL70370||A017166|11||||10^Position^99ABT
    SPM|7|||NotParticipating^NotParticipating^HL70487|||||||P^Patient^HL70369
    SAC|||634013322041|||||I^Identified^HL70370||A017166|12||||10^Position^99ABT
    SPM|8|||NotParticipating^NotParticipating^HL70487|||||||P^Patient^HL70369
    """
    )
    expected = {"context": "result", "samples": []}
    assert worklist == expected


def test_decode_sample_in_process(self):
    worklist = self.decode(
        """
    MSH|^~\\&|ABBOTT_IVD|ABBOTT|OM_LAB_ANALYZER_MGR|IHE|20130423133433||SSU^U03^SSU_U03|732432a5-6f0d-4e75-bb83-403330d11509|P|2.5.1|||NE|AL||UNICODE UTF-8
    SAC|||123456|||||O^In Process^HL70370||Y195543|1||||2^Position^99ABT
    SPM|1|||""|||||||P^Patient^HL70369
    """
    )
    expected = {"context": "result", "samples": []}
    assert worklist == expected


def test_decode_result(self):
    date = datetime.datetime(2017, 2, 2, 16, 28, 54)
    worklist = self.decode(
        """
    MSH|^~\\&|AbbottOne|Abbott|Abbott|Abbott|20170202162858||OUL^R22^OUL_R22|b8a1f69e-f333-4105-b421-785972bf5f97|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-29^IHE
    SPM|1|||""|||||||P^Patient^HL70369
    SAC|||10|||||||B001345|10||||1^Position^99ABT
    OBR||""||1^BENCH_HBV^99ABT
    ORC|SC||||CM
    TQ1|||||||||R^Routine^HL70485
    OBX|1|SN|1^BENCH_HBV^99ABT|1|4,04|^Log(IU/mL)||RUO^^99ABT~SIM^^99ABT|||F|||||Admin~Admin||MQ^Abbott~A12345^Abbott|20170202162854||||||||||RSLT
    INV|1^^99ABT|NA^Not applicable^HL70383|CA^Calibration^99ABT|||||||||||||800NINE13
    INV|1001-AMP_KIT_REAGENT^^99ABT|NA^Not applicable^HL70383|SR^Single Test Reagent^HL70384|||||||||||||800NINE13
    INV|100-SAMPLE_PREP_REAGENT^^99ABT|NA^Not applicable^HL70383|SR^Single Test Reagent^HL70384|||||||||||||10000TWO3
    INV|LYSIS^^99ABT|NA^Not applicable^HL70383|LI^Measurable Liquid Item^HL70384|||||||||||||L12345678
    INV|ETHANOL^^99ABT|NA^Not applicable^HL70383|LI^Measurable Liquid Item^HL70384|||||||||||||B12345678
    INV|DILUENT^^99ABT|NA^Not applicable^HL70383|LI^Measurable Liquid Item^HL70384|||||||||||||D12345678
    INV|VAPOR_BARRIER^^99ABT|NA^Not applicable^HL70383|LI^Measurable Liquid Item^HL70384|||||||||||||C12345678
    OBX|2|NM|1.R^BENCH_HBV^99ABT^S_RAW^Raw Supplemental^IHELAW|1|26,17|^CN||""|||F|||||Admin~Admin||MQ^Abbott~A12345^Abbott|20170202162854||||||||||RSLT
    OBX|3|EI|1.G^BENCH_HBV^99ABT^S_OTHER^Other Supplemental^IHELAW|1|ff69a5f4-b57b-4d22-aea2-9e7eddba15b0|||""|||F|||||Admin~Admin||MQ^Abbott~A12345^Abbott|20170202162854||||||||||RSLT
    OBX|4|NM|1.M^BENCH_HBV^99ABT^S_OTHER^Other Supplemental^IHELAW|HBV FAM target|0.49|||""|||F|||||Admin~Admin||MQ^Abbott~A12345^Abbott|20170202162854||||||||||RSLT
    OBX|5|NM|1.M^BENCH_HBV^99ABT^S_OTHER^Other Supplemental^IHELAW|HBVIC|0.49|||""|||F|||||Admin~Admin||MQ^Abbott~A12345^Abbott|20170202162854||||||||||RSLT
    OBX|6|NM|1.C^BENCH_HBV^99ABT^S_OTHER^Other Supplemental^IHELAW|HBVIC|26.17|^CN||""|||F|||||Admin~Admin||MQ^Abbott~A12345^Abbott|20170202162854||||||||||RSLT
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "1",
                        "date": date,
                        "instrument": "A12345",
                        "operator": "Admin",
                        "unit": "",
                        "value": "4.04",
                    },
                    {
                        "code": "1.R",
                        "date": date,
                        "instrument": "A12345",
                        "operator": "Admin",
                        "unit": "",
                        "value": "26.17",
                    },
                    {
                        "code": "1.G",
                        "date": date,
                        "instrument": "A12345",
                        "operator": "Admin",
                        "unit": "",
                        "value": "ff69a5f4-b57b-4d22-aea2-9e7eddba15b0",
                    },
                    {
                        "code": "1.M",
                        "date": date,
                        "instrument": "A12345",
                        "operator": "Admin",
                        "unit": "",
                        "value": "0.49",
                    },
                    {
                        "code": "1.M",
                        "date": date,
                        "instrument": "A12345",
                        "operator": "Admin",
                        "unit": "",
                        "value": "0.49",
                    },
                    {
                        "code": "1.C",
                        "date": date,
                        "instrument": "A12345",
                        "operator": "Admin",
                        "unit": "",
                        "value": "26.17",
                    },
                ],
                "sample_id": "10",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_control(self):
    self.configure(alinity_no_lot_qc_inv17=True)
    worklist = self.decode(
        """
    MSH|^~\\&|||||20200310180016+0100||OUL^R22^OUL_R22|c0a63680-fd02-40b0-8da7-391b16ca525b|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-29^IHE
    SPM|1|||""|||||||Q^Control specimen^HL70369
    SAC|||HIV-1 LOW POS CTRL|||||||A017165|2||||12^Position^99ABT
    INV|HIV-1 LOW POS CTRL^^99ABT|NA^Not applicable^HL70383|CO^Assay Control^HL70384|003607^^99ABT||||||||20200311||||494934
    OBR||""||1006^HIV-1^99ABT
    OBX|1|SN|1^BENCH_HBV^99ABT|1|4.04|^Log(IU/mL)||RUO^^99ABT~SIM^^99ABT|||F|||||Admin~Admin||MQ^Abbott~A12345^Abbott|20170202162854||||||||||RSLT
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "1",
                        "date": datetime.datetime(2017, 2, 2, 16, 28, 54),
                        "instrument": "A12345",
                        "operator": "Admin",
                        "unit": "",
                        "value": "4.04",
                    }
                ],
                "sample_id": "494934",
                "control_lot": "494934",
                "control_name": "HIV-1 LOW POS CTRL",
                "type": "control",
            }
        ],
    }
    assert worklist == expected


def test_decode_result_multiplex_reagent(self):
    date = datetime.datetime(2017, 2, 2, 16, 28, 54)
    worklist = self.decode(
        """
    MSH|^~\\&|AbbottOne|Abbott|Abbott|Abbott|20170202162858||OUL^R22^OUL_R22|b8a1f69e-f333-4105-b421-785972bf5f97|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-29^IHE
    SPM|1|||""|||||||P^Patient^HL70369
    SAC|||10|||||||B001345|10||||1^Position^99ABT
    OBR||""||1^BENCH_HBV^99ABT
    ORC|SC||||CM
    TQ1|||||||||R^Routine^HL70485
    OBX|1|SN|1^BENCH_HBV^99ABT|1|4.04|^Log(IU/mL)||RUO^^99ABT~SIM^^99ABT|||F|||||Admin~Admin||MQ^Abbott~A12345^Abbott|20170202162854||||||||||RSLT
    INV|1^^99ABT|NA^Not applicable^HL70383|CA^Calibration^99ABT|||||||||||||800NINE13
    INV|1001-AMP_KIT_REAGENT^^99ABT|NA^Not applicable^HL70383|SR^Single Test Reagent^HL70384|||||||||||||800NINE13
    INV|100-SAMPLE_PREP_REAGENT^^99ABT|NA^Not applicable^HL70383|SR^Single Test Reagent^HL70384|||||||||||||10000TWO3
    INV|LYSIS^^99ABT|NA^Not applicable^HL70383|LI^Measurable Liquid Item^HL70384|||||||||||||L12345678
    INV|ETHANOL^^99ABT|NA^Not applicable^HL70383|LI^Measurable Liquid Item^HL70384|||||||||||||B12345678
    INV|DILUENT^^99ABT|NA^Not applicable^HL70383|LI^Measurable Liquid Item^HL70384|||||||||||||D12345678
    INV|VAPOR_BARRIER^^99ABT|NA^Not applicable^HL70383|LI^Measurable Liquid Item^HL70384|||||||||||||C12345678
    OBX|2|NM|1.R^BENCH_HBV^99ABT^S_RAW^Raw Supplemental^IHELAW|1|26.17|^CN||""|||F|||||Admin~Admin||MQ^Abbott~A12345^Abbott|20170202162854||||||||||RSLT
    OBX|3|EI|1.G^BENCH_HBV^99ABT^S_OTHER^Other Supplemental^IHELAW|1|ff69a5f4-b57b-4d22-aea2-9e7eddba15b0|||""|||F|||||Admin~Admin||MQ^Abbott~A12345^Abbott|20170202162854||||||||||RSLT
    OBX|4|NM|1.M^BENCH_HBV^99ABT^S_OTHER^Other Supplemental^IHELAW|HBV FAM target|0.49|||""|||F|||||Admin~Admin||MQ^Abbott~A12345^Abbott|20170202162854||||||||||RSLT
    OBX|5|NM|1.M^BENCH_HBV^99ABT^S_OTHER^Other Supplemental^IHELAW|HBVIC|0.49|||""|||F|||||Admin~Admin||MQ^Abbott~A12345^Abbott|20170202162854||||||||||RSLT
    OBX|6|NM|1.C^BENCH_HBV^99ABT^S_OTHER^Other Supplemental^IHELAW|HBVIC|26.17|^CN||""|||F|||||Admin~Admin||MQ^Abbott~A12345^Abbott|20170202162854||||||||||RSLT
    OBR||""||2^BENCH_HBV^99ABT
    ORC|SC||||CM
    TQ1|||||||||R^Routine^HL70485
    OBX|1|SN|2^BENCH_HBV^99ABT|1|4.04|^Log(IU/mL)||RUO^^99ABT~SIM^^99ABT|||F|||||Admin~Admin||MQ^Abbott~A12345^Abbott|20170202162854||||||||||RSLT
    INV|2^^99ABT|NA^Not applicable^HL70383|CA^Calibration^99ABT|||||||||||||800NINE13
    INV|1001-AMP_KIT_REAGENT^^99ABT|NA^Not applicable^HL70383|SR^Single Test Reagent^HL70384|||||||||||||800NINE13
    INV|100-SAMPLE_PREP_REAGENT^^99ABT|NA^Not applicable^HL70383|SR^Single Test Reagent^HL70384|||||||||||||10000TWO3
    INV|LYSIS^^99ABT|NA^Not applicable^HL70383|LI^Measurable Liquid Item^HL70384|||||||||||||L12345678
    INV|ETHANOL^^99ABT|NA^Not applicable^HL70383|LI^Measurable Liquid Item^HL70384|||||||||||||B12345678
    INV|DILUENT^^99ABT|NA^Not applicable^HL70383|LI^Measurable Liquid Item^HL70384|||||||||||||D12345678
    INV|VAPOR_BARRIER^^99ABT|NA^Not applicable^HL70383|LI^Measurable Liquid Item^HL70384|||||||||||||C12345678
    OBX|2|NM|2.R^BENCH_HBV^99ABT^S_RAW^Raw Supplemental^IHELAW|1|26.17|^CN||""|||F|||||Admin~Admin||MQ^Abbott~A12345^Abbott|20170202162854||||||||||RSLT
    OBX|3|EI|2.G^BENCH_HBV^99ABT^S_OTHER^Other Supplemental^IHELAW|1|ff69a5f4-b57b-4d22-aea2-9e7eddba15b0|||""|||F|||||Admin~Admin||MQ^Abbott~A12345^Abbott|20170202162854||||||||||RSLT
    OBX|4|NM|2.M^BENCH_HBV^99ABT^S_OTHER^Other Supplemental^IHELAW|HBV FAM target|0.49|||""|||F|||||Admin~Admin||MQ^Abbott~A12345^Abbott|20170202162854||||||||||RSLT
    OBX|5|NM|2.M^BENCH_HBV^99ABT^S_OTHER^Other Supplemental^IHELAW|HBVIC|0.49|||""|||F|||||Admin~Admin||MQ^Abbott~A12345^Abbott|20170202162854||||||||||RSLT
    OBX|6|NM|2.C^BENCH_HBV^99ABT^S_OTHER^Other Supplemental^IHELAW|HBVIC|26.17|^CN||""|||F|||||Admin~Admin||MQ^Abbott~A12345^Abbott|20170202162854||||||||||RSLT
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "1",
                        "date": date,
                        "instrument": "A12345",
                        "operator": "Admin",
                        "unit": "",
                        "value": "4.04",
                    },
                    {
                        "code": "1.R",
                        "date": date,
                        "instrument": "A12345",
                        "operator": "Admin",
                        "unit": "",
                        "value": "26.17",
                    },
                    {
                        "code": "1.G",
                        "date": date,
                        "instrument": "A12345",
                        "operator": "Admin",
                        "unit": "",
                        "value": "ff69a5f4-b57b-4d22-aea2-9e7eddba15b0",
                    },
                    {
                        "code": "1.M",
                        "date": date,
                        "instrument": "A12345",
                        "operator": "Admin",
                        "unit": "",
                        "value": "0.49",
                    },
                    {
                        "code": "1.M",
                        "date": date,
                        "instrument": "A12345",
                        "operator": "Admin",
                        "unit": "",
                        "value": "0.49",
                    },
                    {
                        "code": "1.C",
                        "date": date,
                        "instrument": "A12345",
                        "operator": "Admin",
                        "unit": "",
                        "value": "26.17",
                    },
                    {
                        "code": "2",
                        "date": date,
                        "instrument": "A12345",
                        "operator": "Admin",
                        "unit": "",
                        "value": "4.04",
                    },
                    {
                        "code": "2.R",
                        "date": date,
                        "instrument": "A12345",
                        "operator": "Admin",
                        "unit": "",
                        "value": "26.17",
                    },
                    {
                        "code": "2.G",
                        "date": date,
                        "instrument": "A12345",
                        "operator": "Admin",
                        "unit": "",
                        "value": "ff69a5f4-b57b-4d22-aea2-9e7eddba15b0",
                    },
                    {
                        "code": "2.M",
                        "date": date,
                        "instrument": "A12345",
                        "operator": "Admin",
                        "unit": "",
                        "value": "0.49",
                    },
                    {
                        "code": "2.M",
                        "date": date,
                        "instrument": "A12345",
                        "operator": "Admin",
                        "unit": "",
                        "value": "0.49",
                    },
                    {
                        "code": "2.C",
                        "date": date,
                        "instrument": "A12345",
                        "operator": "Admin",
                        "unit": "",
                        "value": "26.17",
                    },
                ],
                "sample_id": "10",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_result_hpv(self):
    date = datetime.datetime(2023, 4, 1, 13, 9, 51)
    worklist = self.decode(
        """
    MSH|^~\\&|||||20230401131013+0200||OUL^R22^OUL_R22|56063705-ed43-47ee-bd5b-eec0b45eeb82|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-29^IHE
    SPM|1|||LDT^LDT^HL70487|||||||P^Patient^HL70369
    SAC|||643033006144|||||||A075738|3||||6^Position^99ABT
    OBR||1680167591401||2007^HR HPV^99ABT
    ORC|SC||||CM
    TQ1|||||||||R^Routine^HL70485
    OBX|1|ST|2007^HR HPV^99ABT||Other HR HPV A|||LDT^^99ABT|||F|||||THT~THT||Alinity m^Abbott~M00474^Abbott~3^Abbott~5^Abbott|20230401130951||||||||||RSLT
    INV|2007^^99ABT|NA^Not applicable^HL70383|CA^Calibration^99ABT|||||||||||||380933
    INV|HR HPV NEG CTRL^^99ABT|NA^Not applicable^HL70383|CO^Assay Control^HL70384|003532^^99ABT||||||||20240725||||771337
    INV|HR HPV POS CTRL^^99ABT|NA^Not applicable^HL70383|CO^Assay Control^HL70384|007517^^99ABT||||||||20240725||||384022
    INV|5001-AMP_KIT_REAGENT^^99ABT|NA^Not applicable^HL70383|SR^Single Test Reagent^HL70384|01029^^99ABT||||||||20240811||||380933
    INV|5002-AMP_KIT_REAGENT^^99ABT|NA^Not applicable^HL70383|SR^Single Test Reagent^HL70384|00160^^99ABT||||||||20240811||||380933
    INV|100-SAMPLE_PREP_REAGENT^^99ABT|NA^Not applicable^HL70383|SR^Single Test Reagent^HL70384|00141^^99ABT||||||||20230929||||382189
    INV|100-SAMPLE_PREP_REAGENT^^99ABT|NA^Not applicable^HL70383|SR^Single Test Reagent^HL70384|50983^^99ABT||||||||20230929||||382189
    INV|LYSIS^^99ABT|NA^Not applicable^HL70383|LI^Measurable Liquid Item^HL70384|||||||||20230517||||3431589
    INV|ETHANOL^^99ABT|NA^Not applicable^HL70383|LI^Measurable Liquid Item^HL70384|||||||||20231115||||2471589
    INV|DILUENT^^99ABT|NA^Not applicable^HL70383|LI^Measurable Liquid Item^HL70384|||||||||20231012||||3431586
    INV|VAPOR_BARRIER^^99ABT|NA^Not applicable^HL70383|LI^Measurable Liquid Item^HL70384|||||||||20230808||||2471598
    INV|IRU^^99ABT|NA^Not applicable^HL70383|SC^Countable Solid Item^HL70384|||||||||||||795187
    OBX|2|ST|2007.I^HR HPV^99ABT||HR HPV Detected|||""|||F|||||THT~THT||Alinity m^Abbott~M00474^Abbott~3^Abbott~5^Abbott|20230401130951||||||||||RSLT
    OBX|3|NM|2007.R^HR HPV^99ABT^S_OTHER^Other Supplemental^IHELAW|HPV16|-1,00|^CN||""|||F|||||THT~THT||Alinity m^Abbott~M00474^Abbott~3^Abbott~5^Abbott|20230401130951||||||||||RSLT
    OBX|4|NM|2007.R^HR HPV^99ABT^S_OTHER^Other Supplemental^IHELAW|HPV18|-1,00|^CN||""|||F|||||THT~THT||Alinity m^Abbott~M00474^Abbott~3^Abbott~5^Abbott|20230401130951||||||||||RSLT
    OBX|5|NM|2007.R^HR HPV^99ABT^S_OTHER^Other Supplemental^IHELAW|HPV45|-1,00|^CN||""|||F|||||THT~THT||Alinity m^Abbott~M00474^Abbott~3^Abbott~5^Abbott|20230401130951||||||||||RSLT
    OBX|6|NM|2007.R^HR HPV^99ABT^S_OTHER^Other Supplemental^IHELAW|Other HR HPV A|16,52|^CN||""|||F|||||THT~THT||Alinity m^Abbott~M00474^Abbott~3^Abbott~5^Abbott|20230401130951||||||||||RSLT
    OBX|7|NM|2007.R^HR HPV^99ABT^S_OTHER^Other Supplemental^IHELAW|Other HR HPV B|-1,00|^CN||""|||F|||||THT~THT||Alinity m^Abbott~M00474^Abbott~3^Abbott~5^Abbott|20230401130951||||||||||RSLT
    OBX|8|EI|2007.G^HR HPV^99ABT^S_OTHER^Other Supplemental^IHELAW||9f5afdcf-6bd8-493b-a287-f86ae539f7e3|||""|||F|||||THT~THT||Alinity m^Abbott~M00474^Abbott~3^Abbott~5^Abbott|20230401130951||||||||||RSLT
    OBX|9|NM|2007.M^HR HPV^99ABT^S_OTHER^Other Supplemental^IHELAW|HPV16|0,017|||""|||F|||||THT~THT||Alinity m^Abbott~M00474^Abbott~3^Abbott~5^Abbott|20230401130951||||||||||RSLT
    OBX|10|NM|2007.M^HR HPV^99ABT^S_OTHER^Other Supplemental^IHELAW|HPV18|0,024|||""|||F|||||THT~THT||Alinity m^Abbott~M00474^Abbott~3^Abbott~5^Abbott|20230401130951||||||||||RSLT
    OBX|11|NM|2007.M^HR HPV^99ABT^S_OTHER^Other Supplemental^IHELAW|HPV45|0,011|||""|||F|||||THT~THT||Alinity m^Abbott~M00474^Abbott~3^Abbott~5^Abbott|20230401130951||||||||||RSLT
    OBX|12|NM|2007.M^HR HPV^99ABT^S_OTHER^Other Supplemental^IHELAW|Other HR HPV A|0,245|||""|||F|||||THT~THT||Alinity m^Abbott~M00474^Abbott~3^Abbott~5^Abbott|20230401130951||||||||||RSLT
    OBX|13|NM|2007.M^HR HPV^99ABT^S_OTHER^Other Supplemental^IHELAW|Other HR HPV B|0,003|||""|||F|||||THT~THT||Alinity m^Abbott~M00474^Abbott~3^Abbott~5^Abbott|20230401130951||||||||||RSLT
    OBX|14|NM|2007.M^HR HPV^99ABT^S_OTHER^Other Supplemental^IHELAW|CCR|0,172|||""|||F|||||THT~THT||Alinity m^Abbott~M00474^Abbott~3^Abbott~5^Abbott|20230401130951||||||||||RSLT
    OBX|15|NM|2007.C^HR HPV^99ABT^S_OTHER^Other Supplemental^IHELAW|CC|17,58|^CN||""|||F|||||THT~THT||Alinity m^Abbott~M00474^Abbott~3^Abbott~5^Abbott|20230401130951||||||||||RSLT
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "2007",
                        "date": date,
                        "instrument": "M00474",
                        "operator": "THT",
                        "unit": "",
                        "value": "Other HR HPV A",
                    },
                    {
                        "code": "2007.I",
                        "date": date,
                        "instrument": "M00474",
                        "operator": "THT",
                        "unit": "",
                        "value": "HR HPV Detected",
                    },
                    {
                        "code": "2007.R_HPV16",
                        "date": date,
                        "instrument": "M00474",
                        "operator": "THT",
                        "unit": "",
                        "value": "-1.00",
                    },
                    {
                        "code": "2007.R_HPV18",
                        "date": date,
                        "instrument": "M00474",
                        "operator": "THT",
                        "unit": "",
                        "value": "-1.00",
                    },
                    {
                        "code": "2007.R_HPV45",
                        "date": date,
                        "instrument": "M00474",
                        "operator": "THT",
                        "unit": "",
                        "value": "-1.00",
                    },
                    {
                        "code": "2007.R_Other HR HPV A",
                        "date": date,
                        "instrument": "M00474",
                        "operator": "THT",
                        "unit": "",
                        "value": "16.52",
                    },
                    {
                        "code": "2007.R_Other HR HPV B",
                        "date": date,
                        "instrument": "M00474",
                        "operator": "THT",
                        "unit": "",
                        "value": "-1.00",
                    },
                    {
                        "code": "2007.G",
                        "date": date,
                        "instrument": "M00474",
                        "operator": "THT",
                        "unit": "",
                        "value": "9f5afdcf-6bd8-493b-a287-f86ae539f7e3",
                    },
                    {
                        "code": "2007.M_HPV16",
                        "date": date,
                        "instrument": "M00474",
                        "operator": "THT",
                        "unit": "",
                        "value": "0.017",
                    },
                    {
                        "code": "2007.M_HPV18",
                        "date": date,
                        "instrument": "M00474",
                        "operator": "THT",
                        "unit": "",
                        "value": "0.024",
                    },
                    {
                        "code": "2007.M_HPV45",
                        "date": date,
                        "instrument": "M00474",
                        "operator": "THT",
                        "unit": "",
                        "value": "0.011",
                    },
                    {
                        "code": "2007.M_Other HR HPV A",
                        "date": date,
                        "instrument": "M00474",
                        "operator": "THT",
                        "unit": "",
                        "value": "0.245",
                    },
                    {
                        "code": "2007.M_Other HR HPV B",
                        "date": date,
                        "instrument": "M00474",
                        "operator": "THT",
                        "unit": "",
                        "value": "0.003",
                    },
                    {
                        "code": "2007.M_CCR",
                        "date": date,
                        "instrument": "M00474",
                        "operator": "THT",
                        "unit": "",
                        "value": "0.172",
                    },
                    {
                        "code": "2007.C_CC",
                        "date": date,
                        "instrument": "M00474",
                        "operator": "THT",
                        "unit": "",
                        "value": "17.58",
                    },
                ],
                "sample_id": "643033006144",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_base64_image(self):
    date = datetime.datetime(2017, 2, 2, 16, 28, 54)
    worklist = self.decode(
        """
    MSH|^~\\&|AbbottOne|Abbott|Abbott|Abbott|20170202162858||OUL^R22^OUL_R22|b8a1f69e-f333-4105-b421-785972bf5f97|P|2.5.1|||NE|AL||UNICODE UTF-8|||LAB-29^IHE
    SPM|1|||""|||||||P^Patient^HL70369
    SAC|||10|||||||B001345|10||||1^Position^99ABT
    OBR||""||1^BENCH_HBV^99ABT
    ORC|SC||||CM
    TQ1|||||||||R^Routine^HL70485
    OBX|1|SN|1^BENCH_HBV^99ABT|1|4.04|^Log(IU/mL)||RUO^^99ABT~SIM^^99ABT|||F|||||Admin~Admin||MQ^Abbott~A12345^Abbott|20170202162854||||||||||RSLT
    INV|1^^99ABT|NA^Not applicable^HL70383|CA^Calibration^99ABT|||||||||||||800NINE13
    INV|1001-AMP_KIT_REAGENT^^99ABT|NA^Not applicable^HL70383|SR^Single Test Reagent^HL70384|||||||||||||800NINE13
    INV|100-SAMPLE_PREP_REAGENT^^99ABT|NA^Not applicable^HL70383|SR^Single Test Reagent^HL70384|||||||||||||10000TWO3
    INV|LYSIS^^99ABT|NA^Not applicable^HL70383|LI^Measurable Liquid Item^HL70384|||||||||||||L12345678
    INV|ETHANOL^^99ABT|NA^Not applicable^HL70383|LI^Measurable Liquid Item^HL70384|||||||||||||B12345678
    INV|DILUENT^^99ABT|NA^Not applicable^HL70383|LI^Measurable Liquid Item^HL70384|||||||||||||D12345678
    INV|VAPOR_BARRIER^^99ABT|NA^Not applicable^HL70383|LI^Measurable Liquid Item^HL70384|||||||||||||C12345678
    OBX|2|NM|1.R^BENCH_HBV^99ABT^S_RAW^Raw Supplemental^IHELAW|1|26.17|^CN||""|||F|||||Admin~Admin||MQ^Abbott~A12345^Abbott|20170202162854||||||||||RSLT
    OBX|3|EI|1.G^BENCH_HBV^99ABT^S_OTHER^Other Supplemental^IHELAW|1|ff69a5f4-b57b-4d22-aea2-9e7eddba15b0|||""|||F|||||Admin~Admin||MQ^Abbott~A12345^Abbott|20170202162854||||||||||RSLT
    OBX|4|NM|1.M^BENCH_HBV^99ABT^S_OTHER^Other Supplemental^IHELAW|HBV FAM target|0.49|||""|||F|||||Admin~Admin||MQ^Abbott~A12345^Abbott|20170202162854||||||||||RSLT
    OBX|5|NM|1.M^BENCH_HBV^99ABT^S_OTHER^Other Supplemental^IHELAW|HBVIC|0.49|||""|||F|||||Admin~Admin||MQ^Abbott~A12345^Abbott|20170202162854||||||||||RSLT
    OBX|6|NM|1.C^BENCH_HBV^99ABT^S_OTHER^Other Supplemental^IHELAW|HBVIC|26.17|^CN||""|||F|||||Admin~Admin||MQ^Abbott~A12345^Abbott|20170202162854||||||||||RSLT
    OBX|7|ED|1.P^BENCH_HBV^99ABT^S_OTHER^Other Supplemental^IHELAW|BASELINED-Linear-Target_IC|^IM^JPEG^Base64^/9j/4AAQSkZJRgABAQEAYABgAAD/2wBDAAgGBgcGBQgHBwcJCQgKDBQNDAsLDBkSEw8UHRofHh0aHBwgJC4nICIsIxwcKDcpLDAxNDQ0Hyc5PTgyPC4zNDL/2wBDAQkJCQwLDBgNDRgyIRwhMjIyMjIyMjIyMjIyMjIyMjIyMjIyMjIyMjIyMjIyMjIyMjIyMjIyMjIyMjIyMjIyMjL/wAARCAEAAQADASIAAhEBAxEB/8QAHwAAAQUBAQEBAQEAAAAAAAAAAAECAwQFBgcICQoL|^CN||""|||F|||||Admin~Admin||MQ^Abbott~A12345^Abbott|20170202162854||||||||||RSLT
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "1",
                        "date": date,
                        "instrument": "A12345",
                        "operator": "Admin",
                        "unit": "",
                        "value": "4.04",
                    },
                    {
                        "code": "1.R",
                        "date": date,
                        "instrument": "A12345",
                        "operator": "Admin",
                        "unit": "",
                        "value": "26.17",
                    },
                    {
                        "code": "1.G",
                        "date": date,
                        "instrument": "A12345",
                        "operator": "Admin",
                        "unit": "",
                        "value": "ff69a5f4-b57b-4d22-aea2-9e7eddba15b0",
                    },
                    {
                        "code": "1.M",
                        "date": date,
                        "instrument": "A12345",
                        "operator": "Admin",
                        "unit": "",
                        "value": "0.49",
                    },
                    {
                        "code": "1.M",
                        "date": date,
                        "instrument": "A12345",
                        "operator": "Admin",
                        "unit": "",
                        "value": "0.49",
                    },
                    {
                        "code": "1.C",
                        "date": date,
                        "instrument": "A12345",
                        "operator": "Admin",
                        "unit": "",
                        "value": "26.17",
                    },
                    {
                        "code": "1.P",
                        "date": date,
                        "graph": {
                            "data": b"\xff\xd8\xff\xe0\x00\x10JF"
                            b"IF\x00\x01\x01\x01\x00`"
                            b"\x00`\x00\x00\xff\xdb\x00C"
                            b"\x00\x08\x06\x06\x07\x06\x05\x08"
                            b"\x07\x07\x07\t\t\x08\n\x0c"
                            b"\x14\r\x0c\x0b\x0b\x0c\x19\x12"
                            b"\x13\x0f\x14\x1d\x1a\x1f\x1e\x1d"
                            b"\x1a\x1c\x1c $.' \",#\x1c\x1c(7)"
                            b",01444\x1f'9=82<.342\xff\xdb\x00"
                            b"C\x01\t\t\t\x0c\x0b\x0c"
                            b"\x18\r\r\x182!\x1c!222222222222"
                            b"22222222222222222222222222222222"
                            b"222222\xff\xc0\x00\x11\x08\x01"
                            b'\x00\x01\x00\x03\x01"\x00\x02'
                            b"\x11\x01\x03\x11\x01\xff\xc4\x00"
                            b"\x1f\x00\x00\x01\x05\x01\x01\x01"
                            b"\x01\x01\x01\x00\x00\x00\x00\x00"
                            b"\x00\x00\x00\x01\x02\x03\x04\x05"
                            b"\x06\x07\x08\t\n\x0b",
                            "format": "jpeg",
                            "graph_type": "binary",
                        },
                        "instrument": "A12345",
                        "operator": "Admin",
                        "value": "GRAPHE",
                    },
                ],
                "sample_id": "10",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected
