# -*- coding: utf8 -*-
from __future__ import unicode_literals

parser_class = "aqualinkxml"


def test_void_order(self):
    message = self.order(
        worklist=self.void_worklist,
        context={
            "query": self.parse(
                """
            <?xml version="1.0" encoding="UTF-8"?>
                <S:Envelope xmlns:S="http://schemas.xmlsoap.org/soap/envelope/">
                    <S:Body>
                        <ConveyorInitialization xmlns="http://www.ngnydevices.tech/aqualis/3-0">
                            <ClientId> CLIENT_ID </ClientId>
                            <Tube>
                                <Id> TUBE_ID </Id>
                                <Location>
                                    <RackId> RACK_ID </RackId>
                                    <HoleId> HOLE_ID </HoleId>
                                </Location>
                            </Tube>
                        </ConveyorInitialization>
                    </S:Body>
                </S:Envelope>
            """
            )
        },
    )
    expected = """
    <?xml version="1.0" encoding="UTF-8"?>
        <S:Envelope xmlns:S="http://schemas.xmlsoap.org/soap/envelope/">
            <S:Body>
                <ConveyorInitializationResponse xmlns="http://www.ngnydevices.tech/aqualis/3-0">
                    <Result>PrimaryTubeNotFound</Result>
                </ConveyorInitializationResponse>
            </S:Body>
        </S:Envelope>
    """
    assert message == expected


def test_order_request_tests(self):
    context = {
        "query": self.parse(
            """
            <?xml version="1.0" ?>
            <S:Envelope xmlns:S="http://schemas.xmlsoap.org/soap/envelope/" xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
                <SOAP-ENV:Header></SOAP-ENV:Header>
                <S:Body xmlns="http://www.ngnydevices.tech/aqualis/3-0">
                    <GetTests xmlns="http://www.ngnydevices.tech/aqualis/3-0">
                        <ClientId>aqua7109</ClientId>
                        <PrimaryTube>
                            <Id>036666666</Id>
                            <Location><RackId>#IN1</RackId>
                            <HoleId>A1</HoleId></Location>
                        </PrimaryTube>
                    </GetTests>
                </S:Body>
            </S:Envelope>
        """
        ),
        "request_info": "REQUEST_TESTS",
    }
    _order_message = """
    <?xml version="1.0" encoding="UTF-8"?>
    <S:Envelope xmlns:S="http://schemas.xmlsoap.org/soap/envelope/">
      <S:Body>
        <GetTestsResponse xmlns="http://www.ngnydevices.tech/aqualis/3-0">
          <Result>Success</Result>
          <PrimaryTube>
            <Id>123456</Id>
            <Location>
              <RackId/>
              <HoleId/>
            </Location>
          </PrimaryTube>
          <Patient>
            <Id>036666666</Id>
            <FamilyName>NYME</FamilyName>
            <FirstName>ANO</FirstName>
            <Sex>F</Sex>
            <BirthDate>13/05/2011</BirthDate>
          </Patient>
          <SampleSource>
            <Service>LOC</Service>
          </SampleSource>
          <Order>
            <Priority>STAT</Priority>
          </Order>
          <Tests>
            <Test>
              <Id>A</Id>
              <Status>Pending</Status>
            </Test>
            <Test>
              <Id>B</Id>
              <Status>Done</Status>
            </Test>
          </Tests>
        </GetTestsResponse>
      </S:Body>
    </S:Envelope>
    """
    message = self.order(context=context)
    assert message == _order_message


def test_decode_query_initialization(self):
    worklist = self.decode(
        """
    <?xml version="1.0" encoding="UTF-8"?>
    <S:Envelope xmlns:S="http://schemas.xmlsoap.org/soap/envelope/">
        <S:Body>
            <ConveyorInitialization xmlns="http://www.ngnydevices.tech/aqualis/3-0">
                <ClientId> CLIENT_ID </ClientId>
                <Tube>
                    <Id> TUBE_ID </Id>
                    <Location>
                        <RackId> RACK_ID </RackId>
                        <HoleId> HOLE_ID </HoleId>
                    </Location>
                </Tube>
            </ConveyorInitialization>
        </S:Body>
    </S:Envelope>
    """
    )
    expected = {
        "context": "query",
        "samples": [
            {
                "request_info": "REQUEST_INIT",
                "rack": "RACK_ID-HOLE_ID",
                "sample_id": "TUBE_ID",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_query_production(self):
    worklist = self.decode(
        """
    <?xml version="1.0" ?>
    <S:Envelope xmlns:S="http://schemas.xmlsoap.org/soap/envelope/" xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
        <SOAP-ENV:Header></SOAP-ENV:Header>
        <S:Body xmlns="http://www.ngnydevices.tech/aqualis/3-0">
            <GetTests xmlns="http://www.ngnydevices.tech/aqualis/3-0">
                <ClientId>aqua7109</ClientId>
                <PrimaryTube>
                    <Id>491081002003</Id>
                    <Location><RackId>#IN1</RackId>
                    <HoleId>A1</HoleId></Location>
                </PrimaryTube>
            </GetTests>
        </S:Body>
    </S:Envelope>
    """
    )
    expected = {
        "context": "query",
        "samples": [
            {
                "request_info": "REQUEST_TESTS",
                "rack": "#IN1-A1",
                "sample_id": "491081002003",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_query(self):
    worklist = self.decode(
        """
    <?xml version="1.0" encoding="UTF-8"?>
    <S:Envelope xmlns:S="http://schemas.xmlsoap.org/soap/envelope/">
        <S:Body>
            <GetTests xmlns="http://www.ngnydevices.tech/aqualis/3-0">
                <ClientId> CLIENT_ID </ClientId>
                <PrimaryTube>
                    <Id> TUBE_ID </Id>
                    <Location>
                        <RackId> RACK_ID </RackId>
                        <HoleId> HOLE_ID </HoleId>
                    </Location>
                </PrimaryTube>
            </GetTests>
        </S:Body>
    </S:Envelope>
    """
    )
    expected = {
        "context": "query",
        "samples": [
            {
                "request_info": "REQUEST_TESTS",
                "rack": "RACK_ID-HOLE_ID",
                "sample_id": "TUBE_ID",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_result_secondary_tube(self):
    worklist = self.decode(
        """
    <?xml version="1.0" ?>
    <S:Envelope xmlns:S="http://schemas.xmlsoap.org/soap/envelope/" xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
        <S:Body xmlns="http://www.ngnydevices.tech/aqualis/3-0">
            <SendResults xmlns="http://www.ngnydevices.tech/aqualis/3-0">
                <ClientId>aqua7109</ClientId>
                <ProcessedPrimaryTube>
                    <Id>201812071254</Id>
                    <Status>Success</Status>
                    <Location>
                        <RackId>#ATELLICA1</RackId>
                        <HoleId>B1</HoleId>
                    </Location>
                    <VisualAnalysis>
                        <Width>13.120000000000001</Width>
                        <Height>70.72</Height>
                        <VolumeEstimation>0.0</VolumeEstimation>
                        <CapType>Invalid</CapType>
                        <PictureUrl>http://192.168.23.133:8000/tubes/201812071254.jpg</PictureUrl>
                    </VisualAnalysis>
                </ProcessedPrimaryTube>
                <TestResults>
                    <Test>
                        <Id>AL101</Id>
                        <Status>Success</Status>
                    </Test>
                    <Test>
                        <Id>TRI10</Id>
                        <Status>Success</Status>
                    </Test>
                </TestResults>
                <GeneratedSecondaryTubes>
                    <SecondaryTube>
                        <Id>201812071254</Id>
                        <Location>
                            <RackId>1140</RackId>
                            <HoleId>A2</HoleId>
                        </Location>
                        <Comment></Comment>
                        <VolumeMl>0.5</VolumeMl>
                        <Status>Success</Status>
                        <RelatedTests>
                            <Test>
                                <Id>AL101</Id>
                            </Test>
                        </RelatedTests>
                    </SecondaryTube>
                </GeneratedSecondaryTubes>
            </SendResults>
        </S:Body>
    </S:Envelope>
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "201812071254",
                "rack": "#ATELLICA1-B1",
                "results": [
                    {
                        "code": "AL101",
                        "flags": [
                            "Width:13.120000000000001",
                            "Height:70.72",
                            "Volume:0.0",
                            "CapType:Invalid",
                            "1140-A2",
                        ],
                        "value": "Success",
                    },
                    {
                        "code": "TRI10",
                        "flags": [
                            "Width:13.120000000000001",
                            "Height:70.72",
                            "Volume:0.0",
                            "CapType:Invalid",
                        ],
                        "value": "Success",
                    },
                    {"code": "RACK_AL101", "value": "1140-A2"},
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_result(self):
    worklist = self.decode(
        """
    <?xml version="1.0" encoding="UTF-8"?>
    <S:Envelope xmlns:S="http://schemas.xmlsoap.org/soap/envelope/">
        <S:Body>
            <SendResults xmlns="http://www.ngnydevices.tech/aqualis/3-0">
                <ClientId> CLIENT_ID </ClientId>
                <ProcessedPrimaryTube>
                    <Id> TUBE_ID </Id>
                    <Status> PRIMARY_STATUS </Status>
                    <Location>
                        <RackId> RACK_ID </RackId>
                        <HoleId> HOLE_ID </HoleId>
                    </Location>
                    <VisualAnalysis>
                        <Width> PRIMARY_WIDTH </Width>
                        <Height> PRIMARY_HEIGHT </Height>
                        <VolumeEstimation> VOLUME_ESTIMATION </VolumeEstimation>
                        <CapType> CAP_TYPE </CapType>
                        <HValue> H_VALUE </HValue>
                        <IValue> I_VALUE </IValue>
                        <LValue> L_VALUE </LValue>
                        <PictureUrl> PICTURE_URL </PictureUrl>
                    </VisualAnalysis>
                    <Comment> PRIMARY_COMMENT </Comment>
                </ProcessedPrimaryTube>
                <TestResults>
                    <Test>
                        <Id>TEST1</Id>
                        <Status> TEST_ACTION_RESULT </Status>
                    </Test>
                    <Test>
                        <Id>TESTN</Id>
                        <Status> TEST_ACTION_RESULT </Status>
                    </Test>
                </TestResults>
                <GeneratedSecondaryTubes>
                    <SecondaryTube>
                        <Id> SEC_TUBE_ID </Id>
                        <Location>
                            <RackId> RACK_ID </RackId>
                            <HoleId> HOLE_ID </HoleId>
                        </Location>
                        <Comment> SECONDARY_COMMENT </Comment>
                        <VolumeMl> SECONDARY_VOLUME </VolumeMl>
                        <Status> SECONDARY_STATUS </Status>
                    </SecondaryTube>
                </GeneratedSecondaryTubes>
            </SendResults>
        </S:Body>
    </S:Envelope>
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "TUBE_ID",
                "rack": "RACK_ID-HOLE_ID",
                "results": [
                    {
                        "code": "TEST1",
                        "value": "TEST_ACTION_RESULT",
                        "flags": [
                            "Width:PRIMARY_WIDTH",
                            "Height:PRIMARY_HEIGHT",
                            "Volume:VOLUME_ESTIMATION",
                            "CapType:CAP_TYPE",
                            "Hemolyse:H_VALUE",
                            "Ictere:I_VALUE",
                            "Lactate:L_VALUE",
                        ],
                    },
                    {
                        "code": "TESTN",
                        "value": "TEST_ACTION_RESULT",
                        "flags": [
                            "Width:PRIMARY_WIDTH",
                            "Height:PRIMARY_HEIGHT",
                            "Volume:VOLUME_ESTIMATION",
                            "CapType:CAP_TYPE",
                            "Hemolyse:H_VALUE",
                            "Ictere:I_VALUE",
                            "Lactate:L_VALUE",
                        ],
                    },
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_result_error(self):
    worklist = self.decode(
        """
    <?xml version="1.0" ?>
            <S:Envelope xmlns:S="http://schemas.xmlsoap.org/soap/envelope/" xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
                <SOAP-ENV:Header></SOAP-ENV:Header><S:Body xmlns="http://www.ngnydevices.tech/aqualis/3-0">
                <SendResults xmlns="http://www.ngnydevices.tech/aqualis/3-0">
                <ClientId>aqua7109</ClientId>
                    <ProcessedPrimaryTube><Id>491090205107</Id>
                        <Status>Success</Status>
                        <Location><RackId>#ERROR</RackId><HoleId>B2</HoleId></Location>
                    </ProcessedPrimaryTube>
                    <TestResults><Test><Id>TRI10</Id>
                        <Status>Success</Status>
                    </Test></TestResults>
                <GeneratedSecondaryTubes></GeneratedSecondaryTubes>
                </SendResults>
    </S:Body></S:Envelope>
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "491090205107",
                "rack": "#ERROR-B2",
                "results": [{"code": "TRI10", "value": "Success", "flags": []}],
            }
        ],
    }
    assert worklist == expected


def test_decode_result_option(self):
    self.configure(code_racks="SER,AL,MED")
    worklist = self.decode(
        """
    <?xml version="1.0" ?>
    <S:Envelope xmlns:S="http://schemas.xmlsoap.org/soap/envelope/" xmlns:SOAP-ENV="http://schemas.xmlsoap.org/soap/envelope/">
    <SOAP-ENV:Header></SOAP-ENV:Header><S:Body xmlns="http://www.ngnydevices.tech/aqualis/3-0">
    <SendResults xmlns="http://www.ngnydevices.tech/aqualis/3-0">
    <ClientId>aqua7111</ClientId>
    <ProcessedPrimaryTube><Id>321031115084</Id>
        <Status>Success</Status>
        <Location><RackId>MED00741</RackId><HoleId>15</HoleId></Location>
        <VisualAnalysis><Width>12.39</Width><Height>95.78</Height><VolumeEstimation>0.0</VolumeEstimation>
        <CapType>Gray</CapType><PictureUrl>http://10.1.58.20:8000/tubes/381031114103.jpg</PictureUrl></VisualAnalysis>
    </ProcessedPrimaryTube>
    <TestResults><Test><Id>TRISERO10</Id><Status>Success</Status></Test>
    </TestResults>
    <GeneratedSecondaryTubes></GeneratedSecondaryTubes>
    </SendResults>
    </S:Body></S:Envelope>
    """
    )
    expected = {
        "samples": [
            {
                "type": "patient",
                "sample_id": "321031115084",
                "rack": "MED00741-15",
                "results": [
                    {
                        "code": "TRISERO10",
                        "value": "Success",
                        "flags": [
                            "Width:12.39",
                            "Height:95.78",
                            "Volume:0.0",
                            "CapType:Gray",
                        ],
                    },
                    {"code": "RACK_MED", "value": "MED00741-15"},
                ],
            }
        ],
        "context": "result",
    }
    assert worklist == expected
