# -*- coding: utf8 -*-
from __future__ import unicode_literals

import codecs
import datetime
import imghdr
import os
import copy

from six import BytesIO

parser_class = "bc6800"


def test_decode_query(self):
    worklist = self.decode(
        """
    H|\\^&|1||Mindray^BC-6800^||||||Automated Count^00001|P|LIS2-A2|20150217170247
    Q|1|123456||||20140328103119||||BL
    L|1|N
    """
    )
    expected = {
        "context": "query",
        "samples": [{"type": "patient", "sample_id": "123456"}],
    }
    assert worklist == expected


def test_encode_order_labexpert(self):
    self.configure(protocole_labexpert=True)

    worklist = copy.deepcopy(self.order_worklist)
    worklist.samples[0].priority = "S"
    message = self.order(worklist=worklist)

    expected = """
    H|\\^&|||Mindray^BC-6800||||||Worksheet response^00011|P|LIS2-A2|20130715141912
    P|1|||L036666666|ANO^NYME||20110513121500|Féminin
    O|1|123456|||S|20130603110342|20130513114242|||||||||PRESCRIBER|||||||||Q
    R|1|^Take Mode^^08001|A
    R|2|^Blood Mode^^08002|Serum
    R|3|^Test Mode^^08003
    R|4|^Ref Group^^01002
    R|5|^Remark^^01001
    R|6|^Charge type^^01015
    R|7|^Patient type^^01016
    R|8|^Project Type^^05007|BL
    R|9|^Custom patient Info 1^^01009
    R|10|^Custom patient Info 2^^01010
    R|11|^Custom patient Info 3^^01011
    L|1|N
    """
    assert message == expected


def test_encode_order(self):
    message = self.order()
    expected = """
    H|\\^&|||Mindray^BC-6800||||||Worksheet response^00011|P|LIS2-A2|20130715141912
    P|1|||L036666666|ANO^NYME||20110513121500|Féminin
    O|1|123456||||20130603110342|20130513114242|||||||||PRESCRIBER|||||||||Q
    R|1|^Take Mode^^08001|A
    R|2|^Blood Mode^^08002|Serum
    R|3|^Test Mode^^08003
    R|4|^Ref Group^^01002
    R|5|^Remark^^01001
    R|6|^Charge type^^01015
    R|7|^Patient type^^01016
    R|8|^Project Type^^05007|BL
    R|9|^Custom patient Info 1^^01009
    R|10|^Custom patient Info 2^^01010
    R|11|^Custom patient Info 3^^01011
    L|1|N
    """
    assert message == expected


def test_decode_result_labexpert(self):
    self.configure(protocole_labexpert=True)
    date_result = datetime.datetime(2015, 3, 11, 16, 0, 9)
    worklist = self.decode(
        """
    H|\\^&|1||Mindray^BC-6800^||||||Automated Count^00001|P|LIS2-A2|20150217170247
    P|1|||patientID2001|Michael^Jordan||20081229160009^5^Y|Male||||||||||||||||Internal medicine|A - 501^1002
    O|1|40139349110||||20140805085635|20140705160009|||Jack|||Virus infections|20140716160009|Venous blood^|admin||||||20150311160009|||F
    R|1|^Take Mode^^08001|A|||||||XAV||20150311160009
    C|1|P|Commentaire Practice|G
    R|2|^Blood Mode^^08002|W||^|||||XAV||20150311160009
    C|1|L|Commentaire System|I
    R|3|^Test Mode^^08003|CBC+DIFF||^|||||XAV||20150311160009
    C|1|I|Commentaire Clinical|P
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "40139349110",
                "results": [
                    {
                        "code": "TakeMode",
                        "comments": ["Commentaire Practice"],
                        "flags": [],
                        "value": "A",
                        "date": date_result,
                        "instrument": None,
                        "operator": "XAV",
                    },
                    {
                        "code": "BloodMode",
                        "comments": ["Commentaire System"],
                        "flags": [],
                        "value": "W",
                        "date": date_result,
                        "instrument": None,
                        "operator": "XAV",
                    },
                    {
                        "code": "TestMode",
                        "comments": ["Commentaire Clinical"],
                        "flags": [],
                        "value": "CBC+DIFF",
                        "date": date_result,
                        "instrument": None,
                        "operator": "XAV",
                    },
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_result(self):
    date = datetime.datetime(2015, 3, 11, 16, 0, 9)
    worklist = self.decode(
        """
    H|\\^&|1||Mindray^BC-6800^||||||Automated Count^00001|P|LIS2-A2|20150217170247
    P|1|||patientID2001|Michael^Jordan||20081229160009^5^Y|Male||||||||||||||||Internal medicine|A - 501^1002
    O|1|40139349110||||20140805085635|20140705160009|||Jack|||Virus infections|20140716160009|Venous blood^|admin||||||20150311160009|||F
    R|1|^Take Mode^^08001|A
    R|2|^Blood Mode^^08002|W||^|
    R|3|^Test Mode^^08003|CBC+DIFF||^|
    R|4|^Ref Group^^01002|Femme >15||^|
    R|5|^Remark^^01001|||^|
    R|6|^Recheck flag^^01006|T||^|
    R|7|^Shelf No^^01012|9||^|
    R|8|^Tube No^^01013|8
    R|9|^Charge type^^01015|||^|
    R|10|^Patient type^^01016|||^|
    R|11|^Instrument Name^^08006|BC-6800 #1||^|
    R|12|^Project Type^^05007|BL||^|
    R|13|^WBC^^6690-2|11.34|10&S&9/L|4.00^12.00|H^^N
    R|14|^BAS#^^704-7|0.06|10&S&9/L|0.00^0.10|^^A
    R|15|^BAS%^^706-2|0.4|%|0.0^1.0|^^A
    R|16|^NEU#^^751-8|7.93|10&S&3/uL|1.70^8.20|^^N^^^^
    R|17|^NEU%^^770-8|76.6|%|50.0^70.0|H
    R|18|^EOS#^^711-2|0.33|10&S&3/uL|0.03^0.80|^^N^^^^
    R|19|^EOS%^^713-8|0.1|%|0.5^5.0|L^^A
    R|20|^LYM#^^731-0|2.28|10&S&3/uL|1.10^4.20|^^N^^^^
    R|21|^LYM%^^736-9|20.1|%|20.0^40.0|^^N^^^^
    R|22|^MON#^^742-7|0.78|10&S&3/uL|0.15^0.90|^^N^^^^
    R|23|^MON%^^5905-5|6.9|%|3.0^12.0|^^N^^^^
    R|24|^RBC^^789-8|2.72|10&S&12/L|3.50^5.20|L^^N
    R|25|^HGB^^718-7|8.8|g/dL|12.0^16.0|L^^A
    R|26|^MCV^^787-2|129.8|fL|80.0^100.0|H^^N
    R|27|^MCH^^785-6|32.2|pg|27.0^34.0|^^A
    R|28|^MCHC^^786-4|24.8|g/dL|31.0^37.0|L^^A
    R|29|^RDW-CV^^788-0|24.8|%|11.0^16.0|H^^N
    R|30|^RDW-SD^^21000-5|49.8|fL|35.0^56.0|^^N^^^^
    R|31|^HCT^^4544-3|0.354||0.350^0.490|^^N
    R|32|^PLT^^777-3|55|10&S&9/L|100^300|L^^N
    R|33|^MPV^^32623-1|11.7|fL|6.5^12.0|^^N
    R|34|^PDW^^32207-3|17.2||15.0^17.0|H^^N
    R|35|^PCT^^10002|0.064|%|0.108^0.282|L^^N
    R|36|^PLCR^^10014|31.0|%|11.0^45.0|^^N^^^^
    R|37|^PLCC^^10013|102|10&S&3/uL|30^90|H^^N^^^^
    R|38|^HFC#^^10020|0.00|10&S&3/uL|^|^^N^^^^
    R|39|^HFC%^^10021|0.0|%|^|^^N^^^^
    R|40|^PLT-I^^10022|330|10&S&3/uL|^|^^N^^^^
    R|41|^WBC-D^^10024|11.53|10&S&3/uL|^|^^N^^^^
    R|42|^WBC-B^^10025|11.34|10&S&3/uL|^|^^N^^^^
    R|43|^PDW-SD^^10031|12.9|fL|^|^^N^^^^
    R|44|^InR#^^10032|0.00|10&S&3/uL|^|^^N^^^^
    R|45|^InR%^^10033|0.00|‰|^|^^N^^^^
    R|46|^WBC^^12227-5|11.34|10&S&3/uL|4.00^10.50|H^^N^^^^
    R|47|^IMG#^^51584-1|0.01|10&S&3/uL|0.00^999.99|^^N^^^^
    R|48|^IMG%^^38518-7|0.1|%|0.0^100.0|^^N^^^^
    R|49|^Micro#^^15199-3|0.15|10&S&6/uL|^|^^N^^^^
    R|50|^Micro%^^10042|3.4|%|^|^^N^^^^
    R|51|^Macro#^^15198-5|0.05|10&S&6/uL|^|^^N^^^^
    R|52|^Macro%^^10040|1.2|%|^|^^N^^^^
    R|53|^Neutrophilia^^12004|T
    R|54|^Anisocytosis^^15150-6|T
    R|55|^Macrocytes^^12075|T
    R|56|^Anemia^^12014|T
    R|57|^RBC Histogram. Left Line^^15051|28||^|^^^^^^
    R|58|^RBC Histogram. Right Line^^15052|146||^|^^^^^^
    R|59|^RBC Histogram. Binary Meta Length^^15053|1||^|^^^^^^
    R|60|^RBC Histogram. Total^^15057|74359||^|^^^^^^
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "40139349110",
                "rack": "9-8",
                "results": [
                    {
                        "code": "TakeMode",
                        "comments": [],
                        "flags": [],
                        "value": "A",
                        "date": date,
                        "instrument": None,
                        "operator": "",
                    },
                    {
                        "code": "BloodMode",
                        "comments": [],
                        "flags": [],
                        "value": "W",
                        "date": date,
                        "instrument": None,
                        "operator": "",
                    },
                    {
                        "code": "TestMode",
                        "comments": [],
                        "flags": [],
                        "value": "CBC+DIFF",
                        "date": date,
                        "instrument": None,
                        "operator": "",
                    },
                    {
                        "code": "RefGroup",
                        "comments": [],
                        "flags": [],
                        "value": "Femme >15",
                        "date": date,
                        "instrument": None,
                        "operator": "",
                    },
                    {
                        "code": "Remark",
                        "comments": [],
                        "flags": [],
                        "value": "",
                        "date": date,
                        "instrument": None,
                        "operator": "",
                    },
                    {
                        "code": "Recheckflag",
                        "comments": [],
                        "flags": [],
                        "value": "T",
                        "date": date,
                        "instrument": None,
                        "operator": "",
                    },
                    {
                        "code": "ShelfNo",
                        "comments": [],
                        "flags": [],
                        "value": "9",
                        "date": date,
                        "instrument": None,
                        "operator": "",
                    },
                    {
                        "code": "TubeNo",
                        "comments": [],
                        "flags": [],
                        "value": "8",
                        "date": date,
                        "instrument": None,
                        "operator": "",
                    },
                    {
                        "code": "Chargetype",
                        "comments": [],
                        "flags": [],
                        "value": "",
                        "date": date,
                        "instrument": None,
                        "operator": "",
                    },
                    {
                        "code": "Patienttype",
                        "comments": [],
                        "flags": [],
                        "value": "",
                        "date": date,
                        "instrument": None,
                        "operator": "",
                    },
                    {
                        "code": "InstrumentName",
                        "comments": [],
                        "flags": [],
                        "value": "BC-6800 #1",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "ProjectType",
                        "comments": [],
                        "flags": [],
                        "value": "BL",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "WBC",
                        "comments": [],
                        "flags": ["H"],
                        "value": "11.34",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "BAS#",
                        "comments": [],
                        "flags": ["A"],
                        "value": "0.06",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "BAS%",
                        "comments": [],
                        "flags": ["A"],
                        "value": "0.4",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "NEU#",
                        "comments": [],
                        "flags": [],
                        "value": "7.93",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "NEU%",
                        "comments": [],
                        "flags": ["H"],
                        "value": "76.6",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "EOS#",
                        "comments": [],
                        "flags": [],
                        "value": "0.33",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "EOS%",
                        "comments": [],
                        "flags": ["L", "A"],
                        "value": "0.1",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "LYM#",
                        "comments": [],
                        "flags": [],
                        "value": "2.28",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "LYM%",
                        "comments": [],
                        "flags": [],
                        "value": "20.1",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "MON#",
                        "comments": [],
                        "flags": [],
                        "value": "0.78",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "MON%",
                        "comments": [],
                        "flags": [],
                        "value": "6.9",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "RBC",
                        "comments": [],
                        "flags": ["L"],
                        "value": "2.72",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "HGB",
                        "comments": [],
                        "flags": ["L", "A"],
                        "value": "8.8",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "MCV",
                        "comments": [],
                        "flags": ["H"],
                        "value": "129.8",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "MCH",
                        "comments": [],
                        "flags": ["A"],
                        "value": "32.2",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "MCHC",
                        "comments": [],
                        "flags": ["L", "A"],
                        "value": "24.8",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "RDW-CV",
                        "comments": [],
                        "flags": ["H"],
                        "value": "24.8",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "RDW-SD",
                        "comments": [],
                        "flags": [],
                        "value": "49.8",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "HCT",
                        "comments": [],
                        "flags": [],
                        "value": "0.354",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "PLT",
                        "comments": [],
                        "flags": ["L"],
                        "value": "55",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "MPV",
                        "comments": [],
                        "flags": [],
                        "value": "11.7",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "PDW",
                        "comments": [],
                        "flags": ["H"],
                        "value": "17.2",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "PCT",
                        "comments": [],
                        "flags": ["L"],
                        "value": "0.064",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "PLCR",
                        "comments": [],
                        "flags": [],
                        "value": "31.0",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "PLCC",
                        "comments": [],
                        "flags": ["H"],
                        "value": "102",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "HFC#",
                        "comments": [],
                        "flags": [],
                        "value": "0.00",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "HFC%",
                        "comments": [],
                        "flags": [],
                        "value": "0.0",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "PLT-I",
                        "comments": [],
                        "flags": [],
                        "value": "330",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "WBC-D",
                        "comments": [],
                        "flags": [],
                        "value": "11.53",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "WBC-B",
                        "comments": [],
                        "flags": [],
                        "value": "11.34",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "PDW-SD",
                        "comments": [],
                        "flags": [],
                        "value": "12.9",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "InR#",
                        "comments": [],
                        "flags": [],
                        "value": "0.00",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "InR%",
                        "comments": [],
                        "flags": [],
                        "value": "0.00",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "WBC",
                        "comments": [],
                        "flags": ["H"],
                        "value": "11.34",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "IMG#",
                        "comments": [],
                        "flags": [],
                        "value": "0.01",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "IMG%",
                        "comments": [],
                        "flags": [],
                        "value": "0.1",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "Micro#",
                        "comments": [],
                        "flags": [],
                        "value": "0.15",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "Micro%",
                        "comments": [],
                        "flags": [],
                        "value": "3.4",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "Macro#",
                        "comments": [],
                        "flags": [],
                        "value": "0.05",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "Macro%",
                        "comments": [],
                        "flags": [],
                        "value": "1.2",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "RBCHistogram.LeftLine",
                        "comments": [],
                        "flags": [],
                        "value": "28",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "RBCHistogram.RightLine",
                        "comments": [],
                        "flags": [],
                        "value": "146",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "RBCHistogram.BinaryMetaLength",
                        "comments": [],
                        "flags": [],
                        "value": "1",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "RBCHistogram.Total",
                        "comments": [],
                        "flags": [],
                        "value": "74359",
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                    {
                        "code": "ALARME",
                        "value": "",
                        "comments": [
                            "Neutrophilia",
                            "Anisocytosis",
                            "Macrocytes",
                            "Anemia",
                        ],
                        "date": date,
                        "instrument": "BC-6800 #1",
                        "operator": "",
                    },
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_control(self):
    worklist = self.decode(
        """
    H|\\^&|2||Mindray^BC-6800^||||||LJQCR^00003|P|LIS2-A2|20140909171830
    O|1|||||20140820201334|||||||||^|admin|||||||||F
    R|1|^Take Mode^^08001|A||^
    R|2|^Blood Mode^^08002|W||^
    R|3|^Test Mode^^08003|CBC+DIFF||^
    R|4|^Qc Level^^05001|H||^
    R|5|^QC test date modify flag^^05002|||^
    R|6|^QC test time modify flag^^05003|||^
    R|7|^Qc valid date^^05004|20141111000000||^
    R|8|^Qc file No^^05005|1||^
    R|9|^Qc lot No^^05006|MB034H||^
    R|10|^Instrument Name^^09001|BC6800 #1||^
    R|11|^WBC^^6690-2|19.50|10&S&9/L|16.44^21.44|^^N
    R|12|^BAS#^^704-7|0.54|10&S&9/L|0.22^0.80|^^N
    R|13|^BAS%^^706-2|2.8|%|1.2^4.2|^^N
    R|14|^NEU#^^751-8|13.08|10&S&9/L|10.71^14.71|^^N
    R|15|^NEU%^^770-8|67.0|%|57.1^77.1|^^N
    L|1|N
    """
    )
    date = datetime.datetime(2014, 8, 20, 20, 13, 34)
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "control",
                "control_lot": "MB034H",
                "sample_id": "MB034H",
                "results": [
                    {
                        "code": "WBC",
                        "value": "19.50",
                        "flags": [],
                        "date": date,
                        "instrument": "BC6800 #1",
                    },
                    {
                        "code": "BAS#",
                        "value": "0.54",
                        "flags": [],
                        "date": date,
                        "instrument": "BC6800 #1",
                    },
                    {
                        "code": "BAS%",
                        "value": "2.8",
                        "flags": [],
                        "date": date,
                        "instrument": "BC6800 #1",
                    },
                    {
                        "code": "NEU#",
                        "value": "13.08",
                        "flags": [],
                        "date": date,
                        "instrument": "BC6800 #1",
                    },
                    {
                        "code": "NEU%",
                        "value": "67.0",
                        "flags": [],
                        "date": date,
                        "instrument": "BC6800 #1",
                    },
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_control2(self):
    worklist = self.decode(
        """
    H|\\^&|2||Mindray^BC-6800^||||||LJQCR^00003|P|LIS2-A2|20140909171830
    O|1|||||20140820201334|||||||||^|admin|||||||||F
    R|1|^Take Mode^^08001|A||^
    R|2|^Blood Mode^^08002|W||^
    R|3|^Test Mode^^08003|CBC+DIFF||^
    R|4|^Qc Level^^05001|H||^
    R|5|^QC test date modify flag^^05002|||^
    R|6|^QC test time modify flag^^05003|||^
    R|7|^Qc valid date^^05004|20141111000000||^
    R|8|^Qc file No^^05005|1||^
    R|9|^Qc lot No^^05006|MB034H||^
    R|10|^Analyzer^^09001|BC 6800 #2||^|^^^^^^
    R|11|^WBC^^6690-2|19.50|10&S&9/L|16.44^21.44|^^N
    R|12|^BAS#^^704-7|0.54|10&S&9/L|0.22^0.80|^^N
    R|13|^BAS%^^706-2|2.8|%|1.2^4.2|^^N
    R|14|^NEU#^^751-8|13.08|10&S&9/L|10.71^14.71|^^N
    R|15|^NEU%^^770-8|67.0|%|57.1^77.1|^^N
    L|1|N
    """
    )
    date = datetime.datetime(2014, 8, 20, 20, 13, 34)
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "control",
                "control_lot": "MB034H",
                "sample_id": "MB034H",
                "results": [
                    {
                        "code": "WBC",
                        "value": "19.50",
                        "flags": [],
                        "date": date,
                        "instrument": "BC 6800 #2",
                    },
                    {
                        "code": "BAS#",
                        "value": "0.54",
                        "flags": [],
                        "date": date,
                        "instrument": "BC 6800 #2",
                    },
                    {
                        "code": "BAS%",
                        "value": "2.8",
                        "flags": [],
                        "date": date,
                        "instrument": "BC 6800 #2",
                    },
                    {
                        "code": "NEU#",
                        "value": "13.08",
                        "flags": [],
                        "date": date,
                        "instrument": "BC 6800 #2",
                    },
                    {
                        "code": "NEU%",
                        "value": "67.0",
                        "flags": [],
                        "date": date,
                        "instrument": "BC 6800 #2",
                    },
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_lab_qc(self):
    self.configure(protocole_labexpert=True)
    worklist = self.decode(
        """
        H|\\^&|73||Mindray^LabXpert^||||||LJ QCR^00003|P|LIS2-A2|20221005092357
    O|1|||||20221005092356|||||||||^|User|||||||||F
    R|1|^Take Mode^^08001|A||^|^^^^^^
    R|2|^Blood Mode^^08002|W||^|^^^^^^
    R|3|^Test Mode^^08003|RET||^|^^^^^^
    R|4|^Qc Level^^05001|H||^|^^^^^^
    R|5|^QC test date modify flag^^05002|||^|^^^^^^
    R|6|^QC test time modify flag^^05003|||^|^^^^^^
    R|7|^Qc valid date^^05004|20221110000000||^|^^^^^^
    R|8|^Qc file No^^05005|6||^|^^^^^^
    R|9|^Qc lot No^^05006|ME0922A-3||^|^^^^^^
    R|10|^Analyzer^^09001|BC-6200||^|^^^^^^
    R|11|^HDW^^10083|37.4|g/L|^|^^N^^^^
    R|12|^HYPO-CH%^^17125|14.5|%|^|^^N^^^^
    R|13|^HYPER-CH%^^17126|1.0|%|^|^^N^^^^
    R|14|^MCHm^^17123|22.7|pg|^|^^N^^^^
    R|15|^IPF^^10041|4.5|%|1.1^8.1|^^N^^^^
    R|16|^IPF#^^10062|23|10&S&9/L|^|^^N^^^^
    R|17|^RET#^^14196-0|190.1|10&S&9/L|130.8^243.6|^^N^^^^
    R|18|^RET%^^4679-7|8.21|%|5.64^10.50|^^N^^^^
    R|19|^IRF^^33516-6|11.9|%|0.0^36.0|^^N^^^^
    R|20|^LFR^^10015|88.1|%|64.0^114.0|^^N^^^^
    R|21|^MFR^^10016|8.1|%|0.0^29.5|^^N^^^^
    R|22|^HFR^^10017|3.8|%|0.5^6.5|^^N^^^^
    R|23|^RHE^^10043|25.0|pg|15.3^35.3|^^N^^^^
    R|24|^MCVr^^17122|88.0|fL|^|^^N^^^^
    R|25|^MCHR^^10082|25.1|pg|^|^^N^^^^
    R|26|^DELTA-CH^^17124|2.4|pg|^|^^N^^^^
    R|27|^PLT-O^^10019|526|10&S&9/L|438^678|^^N^^^^
    R|28|^RBC-O^^10018|2.12|10&S&12/L|1.93^2.43|^^N^^^^
    R|29|^WBC-O^^10051|0.67|10&S&9/L|^|^^N^^^^
    R|30|^RPI^^10060|1.7||^|^^N^^^^
    R|31|^H-IPF^^10061|0.0|%|^|^^N^^^^
    R|32|^MRV^^48706-6|87.7|fL|^|^^N^^^^
    R|33|^FRC#^^10067|0.0201|10&S&12/L|^|^^N^^^^
    R|34|^FRC%^^10068|0.87|%|^|^^N^^^^
    L|1|N
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "control_lot": "ME0922A-3",
                "results": [
                    {
                        "code": "HDW",
                        "date": None,
                        "flags": [],
                        "instrument": "BC-6200",
                        "value": "37.4",
                    },
                    {
                        "code": "HYPO-CH%",
                        "date": None,
                        "flags": [],
                        "instrument": "BC-6200",
                        "value": "14.5",
                    },
                    {
                        "code": "HYPER-CH%",
                        "date": None,
                        "flags": [],
                        "instrument": "BC-6200",
                        "value": "1.0",
                    },
                    {
                        "code": "MCHm",
                        "date": None,
                        "flags": [],
                        "instrument": "BC-6200",
                        "value": "22.7",
                    },
                    {
                        "code": "IPF",
                        "date": None,
                        "flags": [],
                        "instrument": "BC-6200",
                        "value": "4.5",
                    },
                    {
                        "code": "IPF#",
                        "date": None,
                        "flags": [],
                        "instrument": "BC-6200",
                        "value": "23",
                    },
                    {
                        "code": "RET#",
                        "date": None,
                        "flags": [],
                        "instrument": "BC-6200",
                        "value": "190.1",
                    },
                    {
                        "code": "RET%",
                        "date": None,
                        "flags": [],
                        "instrument": "BC-6200",
                        "value": "8.21",
                    },
                    {
                        "code": "IRF",
                        "date": None,
                        "flags": [],
                        "instrument": "BC-6200",
                        "value": "11.9",
                    },
                    {
                        "code": "LFR",
                        "date": None,
                        "flags": [],
                        "instrument": "BC-6200",
                        "value": "88.1",
                    },
                    {
                        "code": "MFR",
                        "date": None,
                        "flags": [],
                        "instrument": "BC-6200",
                        "value": "8.1",
                    },
                    {
                        "code": "HFR",
                        "date": None,
                        "flags": [],
                        "instrument": "BC-6200",
                        "value": "3.8",
                    },
                    {
                        "code": "RHE",
                        "date": None,
                        "flags": [],
                        "instrument": "BC-6200",
                        "value": "25.0",
                    },
                    {
                        "code": "MCVr",
                        "date": None,
                        "flags": [],
                        "instrument": "BC-6200",
                        "value": "88.0",
                    },
                    {
                        "code": "MCHR",
                        "date": None,
                        "flags": [],
                        "instrument": "BC-6200",
                        "value": "25.1",
                    },
                    {
                        "code": "DELTA-CH",
                        "date": None,
                        "flags": [],
                        "instrument": "BC-6200",
                        "value": "2.4",
                    },
                    {
                        "code": "PLT-O",
                        "date": None,
                        "flags": [],
                        "instrument": "BC-6200",
                        "value": "526",
                    },
                    {
                        "code": "RBC-O",
                        "date": None,
                        "flags": [],
                        "instrument": "BC-6200",
                        "value": "2.12",
                    },
                    {
                        "code": "WBC-O",
                        "date": None,
                        "flags": [],
                        "instrument": "BC-6200",
                        "value": "0.67",
                    },
                    {
                        "code": "RPI",
                        "date": None,
                        "flags": [],
                        "instrument": "BC-6200",
                        "value": "1.7",
                    },
                    {
                        "code": "H-IPF",
                        "date": None,
                        "flags": [],
                        "instrument": "BC-6200",
                        "value": "0.0",
                    },
                    {
                        "code": "MRV",
                        "date": None,
                        "flags": [],
                        "instrument": "BC-6200",
                        "value": "87.7",
                    },
                    {
                        "code": "FRC#",
                        "date": None,
                        "flags": [],
                        "instrument": "BC-6200",
                        "value": "0.0201",
                    },
                    {
                        "code": "FRC%",
                        "date": None,
                        "flags": [],
                        "instrument": "BC-6200",
                        "value": "0.87",
                    },
                ],
                "sample_id": "ME0922A-3",
                "type": "control",
            }
        ],
    }
    assert worklist == expected


def get_data_file(filename, decode=True, encoding=None):
    filepath = os.path.join(os.path.dirname(__file__), filename)
    if decode:
        encoding = encoding or "utf8"
        with codecs.open(filepath, encoding=encoding) as f:
            data = f.read()
    else:
        with open(filepath, "rb") as f:
            data = f.read()
    return data


def test_decode_graph(self):
    buf = get_data_file("RESU-C_BC6800-testGraph.TXT")
    worklist = self.parser.decode(buf)
    for r in worklist["samples"][0]["results"]:
        if r.value == "GRAPHE":
            assert imghdr.what(BytesIO(r.graph.data)) == "bmp"
