# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

from driverng.astm import HprimDate, HprimString
from driverng.hl7 import HL7DT, HL7DTM


def test_truncate():
    f = HprimString(0, max_length=2, trunc=True)
    assert f.to_hprim(2 * "A") == "AA"
    assert f.to_hprim(3 * "A") == "AA"


def test_date():
    f = HprimDate(0, fmt_date="%d/%m/%Y")
    assert f.to_python("20140122") == datetime.datetime(2014, 1, 22)
    assert f.to_python("22/01/2014") == datetime.datetime(2014, 1, 22)
    assert f.to_hprim(datetime.datetime(2014, 1, 22)) == "22/01/2014"


def test_hl7_date():
    f = HL7DT(0)
    assert f.to_python("20140122") == datetime.datetime(2014, 1, 22)
    assert f.to_hprim(datetime.datetime(2014, 1, 22)) == "20140122"


def test_hl7_datetime():
    f = HL7DTM(0)
    assert f.to_python("201401221251") == datetime.datetime(2014, 1, 22, 12, 51)
    assert f.to_python("20140122125142") == datetime.datetime(2014, 1, 22, 12, 51, 42)
    assert f.to_hprim(datetime.datetime(2014, 1, 22, 11, 32)) == "20140122113200"
