# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "vscube"

MAPPING_CLASS = "vscube"


def test_query(self):
    worklist = self.decode(
        """
    006A015008280105303432\x10280105303419\x10280105303420\x10740105308673\x10280105303425\x10740105308649\x10280105303428\x10740105308665\x10"""
    )
    expected = {
        "context": "query",
        "samples": [
            {"sample_id": "280105303432", "type": "patient"},
            {"sample_id": "280105303419", "type": "patient"},
            {"sample_id": "280105303420", "type": "patient"},
            {"sample_id": "740105308673", "type": "patient"},
            {"sample_id": "280105303425", "type": "patient"},
            {"sample_id": "740105308649", "type": "patient"},
            {"sample_id": "280105303428", "type": "patient"},
            {"sample_id": "740105308665", "type": "patient"},
        ],
    }
    assert worklist == expected


def test_encode_order(self):
    worklist = self.order()
    expected = "0009015001123456\x10"
    assert worklist == expected


def test_void_order(self):
    message = self.order(
        worklist=self.void_worklist,
        context={
            "query": self.parse(
                """
            006A015008280105303432\x10280105303419\x10280105303420\x10740105308673\x10280105303425\x10740105308649\x10280105303428\x10740105308665\x10
            """
            )
        },
    )
    expected = "0002015000"
    assert message == expected


def test_decode_result(self):
    result = self.decode("0025015101440112189538\x101812130956   7004906A2")
    expected = {
        "context": "result",
        "samples": [
            {
                "control_lot": None,
                "sample_id": "440112189538",
                "rack": "4906-A2",
                "results": [
                    {
                        "code": "VS1",
                        "value": "7",
                        "flags": ["0"],
                        "comments": [],
                        "date": datetime.datetime(2013, 12, 18, 9, 56),
                    }
                ],
                "type": "patient",
            }
        ],
    }
    assert result == expected


def test_decode_qc(self):
    result = self.decode(
        "00320152935   281213365619353064096\x101812130921  64104909A2"
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "control_lot": "19353064096",
                "sample_id": "19353064096",
                "rack": "4909-A2",
                "results": [
                    {
                        "code": "VS1",
                        "value": "64",
                        "flags": ["1", "0"],
                        "comments": [
                            "The VES of the QC measured is within "
                            "the acceptability range"
                        ],
                        "date": datetime.datetime(2013, 12, 18, 9, 21),
                    }
                ],
                "type": "control",
            }
        ],
    }
    assert result == expected
